@@ -1,35 +1,30 @@
 mod config;
-
-use self::config::Config;
-
-use super::Backend;
-use crate::ast;
-use crate::complete;
-use crate::consts::TREESITTER_CMAKE_LANGUAGE;
-use crate::filewatcher;
-use crate::formatting::getformat;
-use crate::gammar::checkerror;
-use crate::gammar::LintConfigInfo;
-use crate::jump;
-use crate::scansubs;
-use crate::semantic_token;
-use crate::semantic_token::LEGEND_TYPE;
-use crate::utils::treehelper;
+#[cfg(test)]
+mod test;
 use std::collections::HashMap;
-use std::path::Path;
-use std::sync::Arc;
-use std::sync::RwLock;
+use std::sync::{Arc, LazyLock, RwLock};
+
 use tokio::sync::Mutex;
-use tower_lsp::jsonrpc::Result;
-use tower_lsp::lsp_types;
+use tower_lsp::jsonrpc::{Error as LspError, Result};
 use tower_lsp::lsp_types::*;
-use tower_lsp::LanguageServer;
+use tower_lsp::{lsp_types, LanguageServer};
 use tree_sitter::Parser;
 
-use once_cell::sync::Lazy;
+use self::config::Config;
+use super::Backend;
+use crate::consts::TREESITTER_CMAKE_LANGUAGE;
+use crate::fileapi::DEFAULT_QUERY;
+use crate::formatting::getformat;
+use crate::gammar::{checkerror, ErrorInformation, LintConfigInfo};
+use crate::semantic_token::LEGEND_TYPE;
+use crate::utils::{did_vcpkg_project, treehelper, VCPKG_LIBS, VCPKG_PREFIX};
+use crate::{
+    ast, complete, document_link, fileapi, filewatcher, hover, jump, scansubs, semantic_token,
+    utils,
+};
 
-pub static BUFFERS_CACHE: Lazy<Arc<Mutex<HashMap<lsp_types::Url, String>>>> =
-    Lazy::new(|| Arc::new(Mutex::new(HashMap::new())));
+pub static BUFFERS_CACHE: LazyLock<Arc<Mutex<HashMap<lsp_types::Url, String>>>> =
+    LazyLock::new(|| Arc::new(Mutex::new(HashMap::new())));
 
 static CLIENT_CAPABILITIES: RwLock<Option<TextDocumentClientCapabilities>> = RwLock::new(None);
 
@@ -55,19 +50,44 @@ pub fn client_support_snippet() -> bool {
 }
 
 impl Backend {
+    async fn path_in_project(&self, path: &str) -> bool {
+        if self.root_path.lock().await.is_none() {
+            return true;
+        }
+
+        // NOTE: not enough good, but is ok
+        !path.starts_with("/usr")
+    }
     async fn publish_diagnostics(&self, uri: Url, context: String, lint_info: LintConfigInfo) {
-        let mut parse = Parser::new();
-        parse.set_language(&TREESITTER_CMAKE_LANGUAGE).unwrap();
-        let thetree = parse.parse(&context, None);
-        let Some(tree) = thetree else {
+        if !self.path_in_project(uri.path()).await {
+            return;
+        }
+
+        let Ok(file_path) = uri.to_file_path() else {
+            tracing::error!("Cannot transport {uri:?} to file_path");
+            self.client
+                .log_message(
+                    MessageType::ERROR,
+                    format!("Cannot transport {uri:?} to file_path"),
+                )
+                .await;
             return;
         };
-        let gammererror = checkerror(Path::new(uri.path()), &context, tree.root_node(), lint_info);
+
+        let gammererror = checkerror(&file_path, &context, lint_info);
         if let Some(diagnoses) = gammererror {
             let mut pusheddiagnoses = vec![];
-            for (start, end, message, severity) in diagnoses.inner {
-                let pointx = lsp_types::Position::new(start.row as u32, start.column as u32);
-                let pointy = lsp_types::Position::new(end.row as u32, end.column as u32);
+            for ErrorInformation {
+                start_point,
+                end_point,
+                message,
+                severity,
+            } in diagnoses.inner
+            {
+                let pointx =
+                    lsp_types::Position::new(start_point.row as u32, start_point.column as u32);
+                let pointy =
+                    lsp_types::Position::new(end_point.row as u32, end_point.column as u32);
                 let range = Range {
                     start: pointx,
                     end: pointy,
@@ -132,22 +152,99 @@ impl LanguageServer for Backend {
                     watch_file.dynamic_registration,
                     watch_file.relative_pattern_support,
                 ) {
-                    if let Some(ref uri) = initial.root_uri {
-                        let path = std::path::Path::new(uri.path())
-                            .join("build")
-                            .join("CMakeCache.txt");
+                    if let Some(ref top_path) = initial
+                        .root_uri
+                        .as_ref()
+                        .and_then(|uri| uri.to_file_path().ok())
+                    {
+                        let path = top_path.join("build").join("CMakeCache.txt");
                         if path.exists() {
                             filewatcher::refresh_error_packages(path);
                         }
+
+                        tracing::info!("find cache-v2 json, start reading the data");
+                        let cache_path = top_path
+                            .join("build")
+                            .join(".cmake")
+                            .join("api")
+                            .join("v1")
+                            .join("reply");
+                        if cache_path.is_dir() {
+                            use std::fs;
+                            if let Ok(entries) = fs::read_dir(cache_path) {
+                                for entry in entries.flatten() {
+                                    let file_path = entry.path();
+                                    if file_path.is_file() {
+                                        let Some(file_name) = file_path.file_name() else {
+                                            continue;
+                                        };
+                                        let file_name = file_name.to_string_lossy().to_string();
+                                        if file_name.starts_with("cache-v2")
+                                            && file_name.ends_with(".json")
+                                        {
+                                            fileapi::update_cache_data(file_path);
+                                            break;
+                                        }
+                                    }
+                                }
+                            }
+                        }
+                        tracing::info!("Finish getting the data in cache-v2 json");
                     }
                 }
             }
         }
 
-        if let Some(ref uri) = initial.root_uri {
-            scansubs::scan_all(uri.path()).await;
+        if let Some(ref vcpkg_root) = initial
+            .root_uri
+            .as_ref()
+            .and_then(|uri| uri.to_file_path().ok())
+        {
+            scansubs::scan_all(&vcpkg_root).await;
             let mut root_path = self.root_path.lock().await;
-            root_path.replace(uri.path().into());
+            root_path.replace(vcpkg_root.to_path_buf());
+
+            let build_dir = vcpkg_root.join("build");
+
+            if build_dir.is_dir() {
+                if let Some(query) = &*DEFAULT_QUERY {
+                    query.write_to_build_dir(build_dir.as_path()).ok();
+                }
+            }
+            if did_vcpkg_project(vcpkg_root) {
+                tracing::info!("This project is vcpkg project, start init vcpkg data");
+                let project_root = vcpkg_root;
+                let vcpkg_installed_path = project_root.join("vcpkg_installed");
+
+                #[cfg(unix)]
+                {
+                    use crate::utils::packagepkgconfig::QUERYSRULES;
+                    // When it is found to be a vcpkg project, the pc will be searched first from the vcpkg download directory.
+                    QUERYSRULES.lock().unwrap().insert(
+                        0,
+                        Box::leak(
+                            format!("{}/*.pc", vcpkg_installed_path.to_str().unwrap())
+                                .into_boxed_str(),
+                        ),
+                    );
+                }
+
+                // add vcpkg prefix
+                VCPKG_PREFIX.lock().unwrap().push(Box::leak(
+                    vcpkg_installed_path
+                        .to_str()
+                        .unwrap()
+                        .to_string()
+                        .into_boxed_str(),
+                ));
+
+                if let Ok(paths) = utils::make_vcpkg_package_search_path(&vcpkg_installed_path) {
+                    let mut vcpkg_libs = VCPKG_LIBS.lock().unwrap();
+                    for t in paths {
+                        vcpkg_libs.push(Box::leak(t.into_boxed_str()))
+                    }
+                }
+            }
         }
 
         set_client_text_document(initial.capabilities.text_document);
@@ -220,6 +317,13 @@ impl LanguageServer for Backend {
                     None
                 },
                 references_provider: Some(OneOf::Left(true)),
+
+                document_link_provider: Some(DocumentLinkOptions {
+                    resolve_provider: Some(true),
+                    work_done_progress_options: WorkDoneProgressOptions {
+                        work_done_progress: Some(false),
+                    },
+                }),
                 ..ServerCapabilities::default()
             },
         })
@@ -232,6 +336,10 @@ impl LanguageServer for Backend {
                     glob_pattern: GlobPattern::String("**/CMakeCache.txt".to_string()),
                     kind: Some(lsp_types::WatchKind::all()),
                 },
+                FileSystemWatcher {
+                    glob_pattern: GlobPattern::String("**/.cmake/api/v1/reply/*.json".to_string()),
+                    kind: Some(lsp_types::WatchKind::all()),
+                },
                 FileSystemWatcher {
                     glob_pattern: GlobPattern::String("**/CMakeLists.txt".to_string()),
                     kind: Some(lsp_types::WatchKind::Create | lsp_types::WatchKind::Delete),
@@ -272,25 +380,44 @@ impl LanguageServer for Backend {
     }
 
     async fn did_change_watched_files(&self, params: DidChangeWatchedFilesParams) {
+        let mut has_cached_changed = false;
         for change in params.changes {
-            if let Some("CMakeLists.txt") = change.uri.path().split('/').last() {
-                let Some(ref path) = *self.root_path.lock().await else {
-                    continue;
-                };
-                scansubs::scan_all(path).await;
+            let Ok(file_path) = change.uri.to_file_path() else {
                 continue;
+            };
+            let Some(file_name) = file_path
+                .file_name()
+                .and_then(|name| name.to_str())
+                .map(|name| name.to_string())
+            else {
+                continue;
+            };
+
+            if file_name.ends_with("json") && file_name.starts_with("cache-v2") {
+                fileapi::update_cache_data(&file_path);
             }
-            self.client
-                .log_message(MessageType::INFO, "CMakeCache changed")
-                .await;
-            if let FileChangeType::DELETED = change.typ {
-                filewatcher::clear_error_packages();
-            } else {
-                let path = change.uri.path();
-                filewatcher::refresh_error_packages(path);
+            if file_name.ends_with("txt") {
+                has_cached_changed = true;
+                if file_name == "CMakeLists.txt" {
+                    let Some(ref path) = *self.root_path.lock().await else {
+                        continue;
+                    };
+                    scansubs::scan_all(path).await;
+                    continue;
+                }
+                self.client
+                    .log_message(MessageType::INFO, "CMakeCache changed")
+                    .await;
+                if let FileChangeType::DELETED = change.typ {
+                    filewatcher::clear_error_packages();
+                } else {
+                    filewatcher::refresh_error_packages(file_path);
+                }
             }
         }
-        self.update_diagnostics().await;
+        if has_cached_changed {
+            self.update_diagnostics().await;
+        }
         self.client
             .log_message(MessageType::INFO, "watched files have changed!")
             .await;
@@ -303,6 +430,18 @@ impl LanguageServer for Backend {
         let context = input.text_document.text.clone();
         let mut storemap = BUFFERS_CACHE.lock().await;
         storemap.entry(uri.clone()).or_insert(context.clone());
+        drop(storemap);
+
+        let file_path = match uri.to_file_path() {
+            Ok(file_path) => file_path,
+            Err(_) => {
+                tracing::error!("Cannot get file_path from {uri:?}");
+                return;
+            }
+        };
+
+        complete::update_cache(&file_path, &context).await;
+        jump::update_cache(&file_path, &context).await;
         self.publish_diagnostics(
             uri,
             context,
@@ -323,6 +462,7 @@ impl LanguageServer for Backend {
         let context = input.content_changes[0].text.clone();
         let mut storemap = BUFFERS_CACHE.lock().await;
         storemap.insert(uri.clone(), context.clone());
+        drop(storemap);
         if context.lines().count() < 500 {
             self.publish_diagnostics(
                 uri,
@@ -344,24 +484,35 @@ impl LanguageServer for Backend {
         let storemap = BUFFERS_CACHE.lock().await;
 
         let has_root = self.root_path.lock().await.is_some();
-        if has_root {
-            scansubs::scan_dir(uri.path()).await;
+        let Some(context) = storemap.get(&uri).cloned() else {
+            self.client
+                .log_message(MessageType::INFO, "file saved!")
+                .await;
+            return;
         };
-
-        if let Some(context) = storemap.get(&uri) {
-            if has_root {
-                complete::update_cache(uri.path(), context).await;
+        drop(storemap);
+        let file_path = match uri.to_file_path() {
+            Ok(file_path) => file_path,
+            Err(_) => {
+                tracing::error!("Cannot get file_path from {uri:?}");
+                return;
             }
-            self.publish_diagnostics(
-                uri,
-                context.to_string(),
-                LintConfigInfo {
-                    use_lint: self.init_info.lock().await.enable_lint,
-                    use_extra_cmake_lint: true,
-                },
-            )
-            .await;
+        };
+        if has_root {
+            scansubs::scan_dir(&file_path).await;
+            complete::update_cache(&file_path, &context).await;
+            jump::update_cache(&file_path, &context).await;
         }
+        self.publish_diagnostics(
+            uri,
+            context.to_string(),
+            LintConfigInfo {
+                use_lint: self.init_info.lock().await.enable_lint,
+                use_extra_cmake_lint: true,
+            },
+        )
+        .await;
+
         self.client
             .log_message(MessageType::INFO, "file saved!")
             .await;
@@ -372,27 +523,23 @@ impl LanguageServer for Backend {
         let uri = params.text_document_position_params.text_document.uri;
         let storemap = BUFFERS_CACHE.lock().await;
         self.client.log_message(MessageType::INFO, "Hovered!").await;
-        //notify_send("test", Type::Error);
-        match storemap.get(&uri) {
-            Some(context) => {
-                let mut parse = Parser::new();
-                parse.set_language(&TREESITTER_CMAKE_LANGUAGE).unwrap();
-                let thetree = parse.parse(context.clone(), None);
-                let tree = thetree.unwrap();
-                let output = treehelper::get_cmake_doc(position, tree.root_node(), context);
-                match output {
-                    Some(context) => Ok(Some(Hover {
-                        contents: HoverContents::Scalar(MarkedString::String(context)),
-                        range: Some(Range {
-                            start: position,
-                            end: position,
-                        }),
-                    })),
-                    None => Ok(None),
-                }
-                //notify_send(context, Type::Error);
-                //Ok(None)
-            }
+        let Some(context) = storemap.get(&uri).cloned() else {
+            return Ok(None);
+        };
+        drop(storemap);
+        let mut parse = Parser::new();
+        parse.set_language(&TREESITTER_CMAKE_LANGUAGE).unwrap();
+        let thetree = parse.parse(context.clone(), None);
+        let tree = thetree.unwrap();
+        let output = hover::get_hovered_doc(position, tree.root_node(), &context).await;
+        match output {
+            Some(context) => Ok(Some(Hover {
+                contents: HoverContents::Scalar(MarkedString::String(context)),
+                range: Some(Range {
+                    start: position,
+                    end: position,
+                }),
+            })),
             None => Ok(None),
         }
     }
@@ -411,12 +558,14 @@ impl LanguageServer for Backend {
         } else {
             1
         };
+        let insert_final_newline = input.options.insert_final_newline.unwrap_or(false);
         match storemap.get(&uri) {
             Some(context) => Ok(getformat(
                 context,
                 &self.client,
                 space_line,
                 input.options.insert_spaces,
+                insert_final_newline,
             )
             .await),
             None => Ok(None),
@@ -439,12 +588,19 @@ impl LanguageServer for Backend {
         let storemap = BUFFERS_CACHE.lock().await;
         let urlconent = storemap.get(&uri).cloned();
         drop(storemap);
+        let file_path = match uri.to_file_path() {
+            Ok(file_path) => file_path,
+            Err(_) => {
+                tracing::error!("Cannot get file_path from {uri:?}");
+                return Err(LspError::internal_error());
+            }
+        };
         match urlconent {
             Some(context) => Ok(complete::getcomplete(
                 &context,
                 location,
                 &self.client,
-                uri.path(),
+                &file_path,
                 self.init_info.lock().await.scan_cmake_in_package,
             )
             .await),
@@ -453,25 +609,22 @@ impl LanguageServer for Backend {
     }
     async fn references(&self, input: ReferenceParams) -> Result<Option<Vec<Location>>> {
         let uri = input.text_document_position.text_document.uri;
-        //println!("{:?}", uri);
         let location = input.text_document_position.position;
         let storemap = BUFFERS_CACHE.lock().await;
-        match storemap.get(&uri) {
-            Some(context) => {
-                let mut parse = Parser::new();
-                parse.set_language(&TREESITTER_CMAKE_LANGUAGE).unwrap();
-                //notify_send(context, Type::Error);
-                Ok(jump::godef(
-                    location,
-                    context,
-                    uri.path().to_string(),
-                    &self.client,
-                    false,
-                )
-                .await)
+        let Some(context) = storemap.get(&uri).cloned() else {
+            return Ok(None);
+        };
+        drop(storemap);
+        let mut parse = Parser::new();
+        parse.set_language(&TREESITTER_CMAKE_LANGUAGE).unwrap();
+        let file_path = match uri.to_file_path() {
+            Ok(file_path) => file_path,
+            Err(_) => {
+                tracing::error!("Cannot get file_path from {uri:?}");
+                return Err(LspError::internal_error());
             }
-            None => Ok(None),
-        }
+        };
+        Ok(jump::godef(location, context.as_str(), &file_path, &self.client, false).await)
     }
     async fn goto_definition(
         &self,
@@ -480,49 +633,44 @@ impl LanguageServer for Backend {
         let uri = input.text_document_position_params.text_document.uri;
         let location = input.text_document_position_params.position;
         let storemap = BUFFERS_CACHE.lock().await;
-        match storemap.get(&uri) {
-            Some(context) => {
-                let mut parse = Parser::new();
-                parse.set_language(&TREESITTER_CMAKE_LANGUAGE).unwrap();
-                let thetree = parse.parse(context.clone(), None);
-                let tree = thetree.unwrap();
-                let origin_selection_range =
-                    treehelper::get_position_range(location, tree.root_node());
+        let Some(context) = storemap.get(&uri).cloned() else {
+            return Ok(None);
+        };
+        drop(storemap);
 
-                //notify_send(context, Type::Error);
-                match jump::godef(
-                    location,
-                    context,
-                    uri.path().to_string(),
-                    &self.client,
-                    true,
-                )
-                .await
-                {
-                    Some(range) => Ok(Some(GotoDefinitionResponse::Link({
-                        range
-                            .iter()
-                            .filter(|input| match origin_selection_range {
-                                Some(origin) => origin != input.range,
-                                None => true,
-                            })
-                            .map(|range| LocationLink {
-                                origin_selection_range,
-                                target_uri: range.uri.clone(),
-                                target_range: range.range,
-                                target_selection_range: range.range,
-                            })
-                            .collect()
-                    }))),
-                    None => Ok(None),
-                }
+        let mut parse = Parser::new();
+        parse.set_language(&TREESITTER_CMAKE_LANGUAGE).unwrap();
+        let thetree = parse.parse(context.clone(), None);
+        let tree = thetree.unwrap();
+        let origin_selection_range = treehelper::get_position_range(location, tree.root_node());
 
-                //Ok(None)
+        let file_path = match uri.to_file_path() {
+            Ok(file_path) => file_path,
+            Err(_) => {
+                tracing::error!("Cannot get file_path from {uri:?}");
+                return Err(LspError::internal_error());
             }
+        };
+        match jump::godef(location, &context, &file_path, &self.client, true).await {
+            Some(range) => Ok(Some(GotoDefinitionResponse::Link({
+                range
+                    .iter()
+                    .filter(|input| match origin_selection_range {
+                        Some(origin) => origin != input.range,
+                        None => true,
+                    })
+                    .map(|range| LocationLink {
+                        origin_selection_range,
+                        target_uri: range.uri.clone(),
+                        target_range: range.range,
+                        target_selection_range: range.range,
+                    })
+                    .collect()
+            }))),
             None => Ok(None),
         }
-        //Ok(None)
     }
+
     async fn document_symbol(
         &self,
         input: DocumentSymbolParams,
@@ -548,4 +696,20 @@ impl LanguageServer for Backend {
             None => Ok(None),
         }
     }
+
+    async fn document_link(&self, input: DocumentLinkParams) -> Result<Option<Vec<DocumentLink>>> {
+        let uri = input.text_document.uri;
+        let file_path = match uri.to_file_path() {
+            Ok(file_path) => file_path,
+            Err(_) => {
+                tracing::error!("Cannot get file_path from {uri:?}");
+                return Err(LspError::internal_error());
+            }
+        };
+        let storemap = BUFFERS_CACHE.lock().await;
+        let Some(context) = storemap.get(&uri) else {
+            return Ok(None);
+        };
+        Ok(document_link::document_link_search(context, file_path))
+    }
 }
