@@ -1,14 +1,14 @@
-use tower_lsp::{
-    lsp_types::{SemanticToken, SemanticTokenType, SemanticTokens, SemanticTokensResult},
-    Client,
-};
+use std::sync::LazyLock;
 
-use once_cell::sync::Lazy;
+use tower_lsp::lsp_types::{
+    SemanticToken, SemanticTokenType, SemanticTokens, SemanticTokensResult,
+};
+use tower_lsp::Client;
 
 use crate::consts::TREESITTER_CMAKE_LANGUAGE;
-
-static NUMBERREGEX: Lazy<regex::Regex> =
-    Lazy::new(|| regex::Regex::new(r"^\d+(?:\.+\d*)?").unwrap());
+use crate::CMakeNodeKinds;
+static NUMBERREGEX: LazyLock<regex::Regex> =
+    LazyLock::new(|| regex::Regex::new(r"^\d+(?:\.+\d*)?").unwrap());
 
 const BOOL_VAL: &[&str] = &["ON", "OFF", "TRUE", "FALSE"];
 const UNIQUE_KEYWORD: &[&str] = &["AND", "NOT"];
@@ -79,7 +79,7 @@ fn sub_tokens(
                 *preline = h as u32;
                 *prestart = x as u32;
             }
-            "variable" => {
+            CMakeNodeKinds::VARIABLE => {
                 let h = child.start_position().row;
                 let x = child.start_position().column;
                 let y = child.end_position().column;
@@ -96,7 +96,7 @@ fn sub_tokens(
                 *preline = h as u32;
                 *prestart = x as u32;
             }
-            "normal_command" => {
+            CMakeNodeKinds::NORMAL_COMMAND => {
                 // NOTE: identifier
                 let Some(id) = child.child(0) else {
                     continue;
@@ -123,7 +123,7 @@ fn sub_tokens(
                 res.append(&mut sub_tokens(child, source, preline, prestart, false));
             }
 
-            "line_comment" => {
+            CMakeNodeKinds::LINE_COMMENT => {
                 let h = child.start_position().row;
                 let x = child.start_position().column;
                 let y = child.end_position().column;
@@ -141,11 +141,11 @@ fn sub_tokens(
                 *prestart = x as u32;
             }
 
-            "endmacro_command"
-            | "endif_command"
-            | "endfunction_command"
-            | "else_command"
-            | "endforeach_command" => {
+            CMakeNodeKinds::ENDMACRO_COMMAND
+            | CMakeNodeKinds::ENDIF_COMMAND
+            | CMakeNodeKinds::ENDFUNCTION_COMMAND
+            | CMakeNodeKinds::ELSE_COMMAND
+            | CMakeNodeKinds::ENDFOREACH_COMMAND => {
                 let Some(id) = child.child(0) else {
                     continue;
                 };
@@ -165,7 +165,7 @@ fn sub_tokens(
                 *preline = h as u32;
                 *prestart = x as u32;
             }
-            "argument_list" => {
+            CMakeNodeKinds::ARGUMENT_LIST => {
                 let mut argument_course = child.walk();
                 let mut is_first_val = !is_if; // NOTE: if is if, not check it
                 for argument in child.children(&mut argument_course) {
@@ -398,15 +398,25 @@ fn sub_tokens(
                 *prestart = x as u32;
                 res.append(&mut sub_tokens(child, source, preline, prestart, false));
             }
-            "body" | "macro_def" | "function_def" | "if_condition" | "if_command"
-            | "elseif_command" | "function_command" | "macro_command" | "foreach_loop"
-            | "foreach_command" | "variable_ref" | "normal_var" | "quoted_element" => {
+            CMakeNodeKinds::BODY
+            | CMakeNodeKinds::MACRO_DEF
+            | CMakeNodeKinds::FUNCTION_DEF
+            | CMakeNodeKinds::IF_CONDITION
+            | CMakeNodeKinds::IF_COMMAND
+            | CMakeNodeKinds::ELSEIF_COMMAND
+            | CMakeNodeKinds::FUNCTION_COMMAND
+            | CMakeNodeKinds::MACRO_COMMAND
+            | CMakeNodeKinds::FOREACH_LOOP
+            | CMakeNodeKinds::FOREACH_COMMAND
+            | CMakeNodeKinds::VARIABLE_REF
+            | CMakeNodeKinds::NORMAL_VAR
+            | CMakeNodeKinds::QUOTED_ELEMENT => {
                 res.append(&mut sub_tokens(
                     child,
                     source,
                     preline,
                     prestart,
-                    child.kind() == "if_command",
+                    child.kind() == CMakeNodeKinds::IF_COMMAND,
                 ));
             }
             _ => {}
